﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Nixory - antispyware tool 
## Version: 1.0.3

##Released under GNU General Public License v2

#Load modules
import pygtk, gtk, gobject, ConfigParser
import threading
from time import sleep, time
pygtk.require('2.0')
from os import path, listdir, remove, stat
from pysqlite2 import dbapi2 as sqlite


#Active shield thread
#This thread will periodically check all enabled profiles for changes
#and scan the ones which have been modified.

gtk.gdk.threads_init()

class ActiveShield:
    def __init__(self):
        #Thread
        self.lock = threading.Lock()
        self.thread = threading.Thread(target=self.run)
        
        #Config parser
        self.globalconfig = ConfigParser.ConfigParser()
        self.profiledict = None
        self.initialProfiles = None
        
        #Common vars
        self.isRunning = False
        self.deftray = None
        self.stopthread = threading.Event()
        self.thread.start() #Run thread
    
    def buildProfiles(self):
        #Build profile list
        self.globalconfig.read("./etc/gconfig.cfg")
        
        #Browser -> tuple list of profiles
        self.profiledict = {}
        firefoxlist = []
        ielist = []
        chromelist = []
        profilenum = 0
        profiles = self.globalconfig.get("USER","profiles").split("||") #Profile format: name, path, browser, enabled
        del profiles[-1]
    
        for profile in profiles:
        
            profile = profile.split('|')
            #Check if enabled
            if bool(int(profile[3])):
                if profile[2] == "firefox": firefoxlist.append((profile[0], profile[1]))
                if profile[2] == "ie": ielist.append((profile[0], profile[1]))
                if profile[2] == "chrome": chromelist.append((profile[0], profile[1]))
                profilenum += 1
        #Build
        self.profiledict["firefox"] = tuple(firefoxlist)
        self.profiledict["chrome"] = tuple(chromelist)
        self.profiledict["ie"] = tuple(ielist) 
    
    def start(self):
        #Enable
        self.isRunning = True
        
    def stop(self):
        #Do nothing
        self.isRunning = False
        
    def terminate(self):
        #Kill thread
        self.stopthread.set()
        
    def setTray(self, trayicon):
        #Set default tray icon
        self.deftray = trayicon
    
    def run(self):
        #Run thread
        while not self.stopthread.isSet():
           sleep(5.0)
           self.lock.acquire()
           if self.isRunning:
               gtk.gdk.threads_enter()
               self.checkScan()
               gtk.gdk.threads_leave()
           self.lock.release()
           
           
    def checkScan(self):
        #We will now check if anything changed since last time
        self.globalconfig.read("./etc/gconfig.cfg")
        
        #We have to make sure that our list of enabled profiles is up to date
        if self.globalconfig.get("USER","profiles") != self.initialProfiles:
            #Rebuild profiledict
            self.buildProfiles()
            self.initialProfiles = self.globalconfig.get("USER","profiles")
            
        #Run scan
        self.runScan()
        
        
           
    
    def runScan(self):
       #This scan is similiar to the winscan module but automatically removes everything it finds
       print "Running Scan"
       self.globalconfig.read("./etc/gconfig.cfg")
       lastmod = open("./etc/lastmod","r")
       lastcont = lastmod.read().split(';')
       lastmod.close()
       
       #Global vars
       totchecked = 0
       totcleaned = 0
       defconfig = ConfigParser.ConfigParser()
       defconfig.read("./etc/black_list.cfg")
       whitelist = self.globalconfig.get("USER","whitelist").split(';')
       blacklist = defconfig.get("BLACKLIST","blacklist").split(';')
       del whitelist[-1]
       heurkeys = defconfig.get("HEUR","keys").split(';')
       
       #FIREFOX PROFILES SCAN
       for firefoxprof in self.profiledict["firefox"]:
         doscan = False
         append = False
         if not self.isRunning:
             break
         elif firefoxprof[1] in lastcont:
             if str(stat(firefoxprof[1]).st_mtime) != lastcont[lastcont.index(firefoxprof[1]) + 1]:
                 doscan = True
         else:
             append = True
             doscan = True
         if doscan:          
          #Check if database is accessible
          try:
            conn = sqlite.connect(firefoxprof[1], timeout=0.0) #We cannot wait if database is locked, or GUI would hang up
            curr = conn.cursor()
            curr.execute("SELECT host, name FROM moz_cookies")
            profcookie = curr.fetchall()
          except:
               pass
          
          else:
              #DO scan here
              for cookie in profcookie:
                 if not self.isRunning: 
                     break
                 else:
                    #Check whether the cookie is present in the whitelist
                    if bool(int(self.globalconfig.get('PREFERENCES', 'white_ac'))) and cookie[0] in whitelist:
                            pass
                    
                    #If not, check if it is in the blacklist
                    elif cookie[0] in blacklist:
                            #Cookie spec: browser, id, type (0=potential), name, host
                            try:
                                details = (cookie[0], cookie[1])
                                curr.execute('DELETE FROM moz_cookies WHERE host=? AND name=?', details)
                                totcleaned += 1
                            except:
                                pass
                    
                    #If heuristic is enabled check keys
                    elif bool(int(self.globalconfig.get('PREFERENCES', 'heur_ac'))):
                           for key in heurkeys:
                               if key in cookie[0]:
                                  #Delete   
                                  try:
                                      details = (cookie[0], cookie[1])
                                      curr.execute('DELETE FROM moz_cookies WHERE host=? AND name=?', details)
                                      totcleaned += 1
                                  except:
                                      pass

                               
                    
                    #Add cookie checked
                    totchecked += 1
                  
                  
              
              #End scanning
              try:
                  conn.commit()
              except:
                  pass
              curr.close()
              conn.close()
              
              if append:
                    lastcont.append(firefoxprof[1])
                    lastcont.append(str(stat(firefoxprof[1]).st_mtime))
                    
              #Update
              lastcont[lastcont.index(firefoxprof[1]) + 1] = str(stat(firefoxprof[1]).st_mtime)
          
          
        
       #GOOGLE CHROME PROFILES SCAN
       for chromeprof in self.profiledict["chrome"]:
         doscan = False
         append = False
         if not self.isRunning:
             break
         elif chromeprof[1] in lastcont:
             if str(stat(chromeprof[1]).st_mtime) != lastcont[lastcont.index(chromeprof[1]) + 1]:
                 doscan = True
         else:
             append = True
             doscan = True
         if doscan:          
          #Try to access database
          try:
            conn = sqlite.connect(chromeprof[1])
            curr = conn.cursor()
            curr.execute("SELECT host_key, name FROM cookies")
            profcookie = curr.fetchall()
          except:
               pass
          
          else:
              #DO scan here
              for cookie in profcookie:
                 if not self.isRunning: 
                     break
                 else:

                    #Check whether the cookie is present in the whitelist
                    if bool(int(self.globalconfig.get('PREFERENCES', 'white_ac'))) and cookie[0] in whitelist:
                            pass
                    
                    #If not, check if it is in the blacklist
                    elif cookie[0] in blacklist:
                            #Cookie spec: browser, id, type (0=potential), name, host
                            try:
                                details = (cookie[0], cookie[1])
                                curr.execute('DELETE FROM cookies WHERE host_key=? AND name=?', details)
                                totcleaned += 1
                            except:
                                pass
                    
                    #If heuristic is enabled check keys
                    elif bool(int(self.globalconfig.get('PREFERENCES', 'heur_ac'))):
                           for key in heurkeys:
                               if key in cookie[0]:
                                  
                                  #Delete
                                  try:
                                      details = (cookie[0], cookie[1])
                                      curr.execute('DELETE FROM cookies WHERE host_key=? AND name=?', details)
                                      totcleaned += 1
                                  except:
                                      pass
                               
                    
                    #Add cookie checked
                    totchecked += 1
                  
                  
              
              #End scanning
              try:
                 conn.commit()
              except:
                  pass
              curr.close()
              conn.close()
              
              if append:
                    lastcont.append(chromeprof[1])
                    lastcont.append(str(stat(chromeprof[1]).st_mtime))
                    
              #Update
              lastcont[lastcont.index(chromeprof[1]) + 1] = str(stat(chromeprof[1]).st_mtime)
          
              
              
       #IE SCAN ROUTINE       
       for ieprof in self.profiledict["ie"]:
         doscan = False
         append = False
         if not self.isRunning:
             break
         elif ieprof[1] in lastcont:
             if str(stat(ieprof[1]).st_mtime) != lastcont[lastcont.index(ieprof[1]) + 1]:
                 doscan = True
         else:
             append = True
             doscan = True
         if doscan:         
          #Check if directory is accessible
          try:
            filelist = listdir(ieprof[1])
          except:
               pass
          
          else:
              #Filter files (only .txt and files)
              delist = []
              for file in filelist:
                  if not path.isfile(ieprof[1] + "\\" + file) or (file[-4:] != ".txt"): 
                      delist.append(file)   #Not a file.txt
              
              for file in delist:   
                 del filelist[filelist.index(file)]
              
              #list of cookies to remove
              removelist = []
              
              #DO scan here
              for cookiefile in filelist:
                 if not self.isRunning: 
                     break
                 else:
                  
                  #Cookie file read
                  try:
                     cookieio = open(ieprof[1] + "\\" + cookiefile, "r")
                  except:
                      #skip file
                      pass
                  else:
                    cookiecont = cookieio.read()
                    cookiecont = cookiecont.split('*\n')
                    del cookiecont[-1]
                    totchecked += len(cookiecont)
                  
                    #analize content
                    for cookie in cookiecont:
                      #vars
                      cookie = cookie.split('\n')
                      cookie[2] = cookie[2].replace("/", "")
                    
                      #Check whether the cookie is present in the whitelist
                      if bool(int(self.globalconfig.get('PREFERENCES', 'white_ac'))) and ((cookie[2] in whitelist) or ('.' + cookie[2] in whitelist)):
                            #Ignore this cookie
                            pass
                    
                      #If not, check if it is in the blacklist
                      #Internet explorer has the bad habit to remove the dot at the beginning of each host, so let's add it
                      elif (cookie[2] in blacklist) or ('.' + cookie[2] in blacklist):
                            #Remove file directly
                            removelist.append(ieprof[1] + "\\" + cookiefile)
                            totcleaned += len(cookiecont)
                            
                            break
                    
                      #If heuristic is enabled check keys
                      elif bool(int(self.globalconfig.get('PREFERENCES', 'heur_ac'))):
                           for key in heurkeys:
                               if key in cookie[2]:
    
                                  #Remove file directly
                                  removelist.append(ieprof[1] + "\\" + cookiefile)
                                  totcleaned += len(cookiecont)
                                  
                                  break
                    

                   
                    #Close connection
                    cookieio.close()
                  
              #Remove listed objs
              for object in removelist:
                      try: remove(object)
                      except: pass
                  
              if append:
                    lastcont.append(ieprof[1])
                    lastcont.append(str(stat(ieprof[1]).st_mtime))
                    
              #Update
              lastcont[lastcont.index(ieprof[1]) + 1] = str(stat(ieprof[1]).st_mtime)

         
       #Update stats
       print totchecked, totcleaned
       self.globalconfig.set("GENERAL","shield_scanned", str(int(self.globalconfig.get("GENERAL","shield_scanned")) + totchecked))
       self.globalconfig.set("GENERAL","shield_cleaned", str(int(self.globalconfig.get("GENERAL","shield_cleaned")) + totcleaned))
       self.globalconfig.write(open("./etc/gconfig.cfg","w"))   
       
       #Update tray if necessary
       if int(self.globalconfig.get("PREFERENCES", "show_tooltip")): self.deftray.set_tooltip("Running:  %s Objects cleaned" % self.globalconfig.get("GENERAL", "shield_cleaned"))
       
       #Write lastmod
       lastmod = open("./etc/lastmod","w")
       lastmod.write(";".join(lastcont))
       lastmod.close()
        

#Common instance
actshield = ActiveShield()    